@@ -1,8 +1,10 @@
 """All types used by funml"""
+from __future__ import annotations
+
 import functools
-from collections.abc import Awaitable
+from collections.abc import Awaitable, Callable
 from inspect import signature, Parameter, Signature
-from typing import Any, Union, Callable, Optional, List, Tuple
+from typing import Any, Optional, List, Tuple, Union
 
 from funml import errors
 from funml.utils import is_equal_or_of_type
@@ -14,14 +16,16 @@ class MLType:
     Methods common to ML-enabled types are defined in this class.
     """
 
-    def generate_case(self, do: "Operation") -> Tuple[Callable, "Expression"]:
+    def generate_case(
+        self, do: "Operation"
+    ) -> Tuple[Callable[[Any], bool], "Expression"]:
         """Generates a case statement for pattern matching.
 
         Args:
             do: The operation to do if the arg matches on this type
 
         Returns:
-            A tuple (checker, expn) where checker is a function that checks if argument matches this case, and expn
+            A tuple (checker, expn) where checker is a function that checks if argument matches this case, and expn \
             is the expression that is called when the case is matched.
         """
         raise NotImplemented("generate_case not implemented")
@@ -50,7 +54,12 @@ class Pipeline:
         self._is_terminated = False
 
     def __rshift__(
-        self, nxt: Union["Expression", Callable, "Pipeline"]
+        self,
+        nxt: Union[
+            "Expression",
+            Callable,
+            "Pipeline",
+        ],
     ) -> Union["Pipeline", Any]:
         """Uses `>>` to append the nxt expression, callable, pipeline to this pipeline.
 
@@ -58,7 +67,7 @@ class Pipeline:
             nxt: the next expression, pipeline, or callable to apply after the current one.
 
         Returns:
-            the updated pipeline or the value when the pipeline is executed in case `nxt` is of
+            the updated pipeline or the value when the pipeline is executed in case `nxt` is of \
             type `ExecutionExpression`
 
         Raises:
@@ -70,7 +79,9 @@ class Pipeline:
 
         return self
 
-    def __call__(self, *args: Any, **kwargs: Any) -> Any:
+    def __call__(
+        self, *args: Any, **kwargs: Any
+    ) -> Union["Expression", Awaitable["Expression"], Awaitable[Any], Any,]:
         """Computes the logic within the pipeline and returns the value.
 
         This method runs all those expressions in the queue sequentially,
@@ -82,7 +93,7 @@ class Pipeline:
             kwargs: any key-word arguments passed
 
         Returns:
-            the computed output of this pipeline.
+            the computed output of this pipeline or a partial expression if the args and kwargs provided are less than those expected.
         """
         output = None
         queue = self._queue[:-1] if self._is_terminated else self._queue
@@ -117,7 +128,14 @@ class Pipeline:
         pipe._is_terminated = self._is_terminated
         return pipe
 
-    def __update_queue(self, nxt):
+    def __update_queue(
+        self,
+        nxt: Union[
+            "Expression",
+            Callable,
+            "Pipeline",
+        ],
+    ):
         """Appends a pipeline or an expression to the queue."""
         if self._is_terminated:
             raise ValueError("a terminated pipeline cannot be extended.")
@@ -137,7 +155,7 @@ class AsyncPipeline(Pipeline):
     See more details in the [base class](funml.types.Pipeline)
     """
 
-    async def __call__(self, *args: Any, **kwargs: Any) -> Any:
+    async def __call__(self, *args: Any, **kwargs: Any) -> Union["Expression", Any]:
         """Computes the logic within the pipeline and returns the value.
 
         This method runs all those expressions in the queue sequentially,
@@ -149,7 +167,8 @@ class AsyncPipeline(Pipeline):
             kwargs: any key-word arguments passed
 
         Returns:
-            the computed output of this pipeline.
+            the computed output of this pipeline or a partial expression if the args and kwargs provided \
+            are less than expected.
         """
         output = None
         queue = self._queue[:-1] if self._is_terminated else self._queue
@@ -165,6 +184,45 @@ class AsyncPipeline(Pipeline):
         return output
 
 
+class Operation:
+    """A computation.
+
+    Args:
+        func: the logic to run as part of the operation.
+    """
+
+    def __init__(self, func: Callable):
+        self.__signature = _get_func_signature(func)
+        self.__args_length = _get_non_variable_args_length(self.__signature)
+
+        if len(self.__signature.parameters) == 0:
+            # be more fault tolerant by using variable params
+            self.__f = lambda *args, **kwargs: func()
+        else:
+            self.__f = func
+
+    def __call__(self, *args: Any, **kwargs: Any) -> Union["Operation", Any]:
+        """Applies the logic attached to this operation and returns output.
+
+        Args:
+            args: the args passed
+            kwargs: the context in which the operation is being run.
+
+        Returns:
+            the final output of the operation's logic code or a partial operation if the args and kwargs \
+            provided are less than those expected.
+        """
+        try:
+            args_length = _get_num_of_relevant_args(self.__signature, *args, **kwargs)
+            if args_length < self.__args_length:
+                return Operation(func=functools.partial(self.__f, *args, **kwargs))
+        except TypeError:
+            # binding is impossible so just use the default implementation
+            pass
+
+        return self.__f(*args, **kwargs)
+
+
 class Expression:
     """Logic that returns a value when applied.
 
@@ -178,7 +236,7 @@ class Expression:
     def __init__(self, f: Optional["Operation"] = None):
         self._f = f if f is not None else Operation(lambda x, *args, **kwargs: x)
 
-    def __call__(self, *args: Any, **kwargs: Any) -> Any:
+    def __call__(self, *args: Any, **kwargs: Any) -> Union["Expression", Any]:
         """Computes the logic within and returns the value.
 
         Args:
@@ -186,12 +244,21 @@ class Expression:
             kwargs: any key-word arguments passed
 
         Returns:
-            the computed output of this expression.
+            the computed output of this expression or another expression with a partial operation \
+            in it if the args provided are less than expected.
         """
-        return self._f(*args, **kwargs)
+        value = self._f(*args, **kwargs)
+        if isinstance(value, Operation):
+            return Expression(f=value)
+        return value
 
     def __rshift__(
-        self, nxt: Union["Expression", "Pipeline", Callable]
+        self,
+        nxt: Union[
+            "Expression",
+            "Pipeline",
+            Callable,
+        ],
     ) -> Union["Pipeline", Any]:
         """This makes piping using the '>>' symbol possible.
 
@@ -202,7 +269,7 @@ class Expression:
             nxt: the next expression, pipeline, or callable to apply after the current one.
 
         Returns:
-            a new pipeline  where the first expression is the current expression followed by `nxt`
+            a new pipeline  where the first expression is the current expression followed by `nxt` \
             or returns the value when the pipeline is executed in case `nxt` is of type `ExecutionExpression`
         """
         new_pipeline = Pipeline()
@@ -235,7 +302,7 @@ class MatchExpression(Expression):
 
     def __init__(self, arg: Optional[Any] = None):
         super().__init__()
-        self._matches: List[Tuple[Callable, Expression]] = []
+        self._matches: List[Tuple[Callable[[Any], bool], Expression[T]]] = []
         self.__arg = arg
 
     def case(self, pattern: Union[MLType, Any], do: Callable) -> "MatchExpression":
@@ -260,7 +327,7 @@ class MatchExpression(Expression):
         self.__add_match(check=check, expn=expn)
         return self
 
-    def __add_match(self, check: Callable, expn: "Expression"):
+    def __add_match(self, check: Callable[[Any], bool], expn: "Expression"):
         """Adds a match set to the list of match sets
 
         A match set comprises a checker function and an expression.
@@ -281,7 +348,7 @@ class MatchExpression(Expression):
 
         self._matches.append((check, expn))
 
-    def __call__(self, arg: Optional[Any] = None) -> Any:
+    def __call__(self, arg: Optional[Any] = None) -> Union["Expression", Any]:
         """Applies the matched case and returns the output.
 
         The match cases are surveyed for any that matches the given argument
@@ -307,44 +374,6 @@ class MatchExpression(Expression):
         raise errors.MatchError(arg)
 
 
-class Operation:
-    """A computation.
-
-    Args:
-        func: the logic to run as part of the operation.
-    """
-
-    def __init__(self, func: Callable):
-        self.__signature = _get_func_signature(func)
-        self.__args_length = _get_non_variable_args_length(self.__signature)
-
-        if len(self.__signature.parameters) == 0:
-            # be more fault tolerant by using variable params
-            self.__f = lambda *args, **kwargs: func()
-        else:
-            self.__f = func
-
-    def __call__(self, *args: Any, **kwargs: Any) -> Any:
-        """Applies the logic attached to this operation and returns output.
-
-        Args:
-            args: the args passed
-            kwargs: the context in which the operation is being run.
-
-        Returns:
-            the final output of the operation's logic code.
-        """
-        try:
-            args_length = _get_num_of_relevant_args(self.__signature, *args, **kwargs)
-            if args_length < self.__args_length:
-                return Operation(func=functools.partial(self.__f, *args, **kwargs))
-        except TypeError:
-            # binding is impossible so just use the default implementation
-            pass
-
-        return self.__f(*args, **kwargs)
-
-
 def _get_num_of_relevant_args(sig: Signature, *args, **kwargs) -> int:
     """Computes the number of args and kwargs relevant to the signature
 
